from browser import document as doc, window, alert
import io
import sys
import traceback

class BrythonException:
    def __init__(self, msg):
        self.exception_type = 'error'
        self.line = None
        self.exception_msg = msg

class ExecutionResult:
    def __init__(self, output, stderr, exception):
        self.output = output
        self.stderr = stderr
        self.exception = exception


def execute_python(source, input_data):
    orig_stdin = sys.stdin
    orig_stdout = sys.stdout
    orig_stderr = sys.stderr

    sys.stdin = io.StringIO(input_data)
    sys.stdout = io.StringIO()
    sys.stderr = io.StringIO()

    try:
        exec(source, {'__name__': '__main__'})
    except Exception as exc:
        traceback.print_exc(file=sys.stderr)
    finally:
        exec_stdout = sys.stdout
        exec_stderr = sys.stderr

        sys.stdin = orig_stdin
        sys.stdout = orig_stdout
        sys.stderr = orig_stderr
    error_msg = exec_stderr.getvalue()
    return {
        'stdout': exec_stdout.getvalue(),
        'stderr': 'runtime_error' if error_msg else '',
        'exception': BrythonException(error_msg) if error_msg != '' else '',
    }


def run_test(user_code, test_input):
    res = execute_python(user_code, test_input)
    user_answer = res['stdout'].strip()

    return ExecutionResult(user_answer, res['stderr'], res['exception'])


window.run_test = run_test
